/*
 * Copyright (c) 2005- Shinji Kashihara.
 * All rights reserved. This program are made available under
 * the terms of the Eclipse Public License v1.0 which accompanies
 * this distribution, and is available at epl-v10.html.
 */
package jp.sourceforge.mergedoc.pleiades.aspect;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.lang.instrument.ClassFileTransformer;
import java.lang.instrument.IllegalClassFormatException;
import java.net.URL;
import java.security.ProtectionDomain;
import java.util.HashSet;
import java.util.Set;

import javassist.CannotCompileException;
import javassist.ClassPool;
import javassist.CtBehavior;
import javassist.CtClass;
import javassist.NotFoundException;
import jp.sourceforge.mergedoc.pleiades.log.Logger;

/**
 * oCgR[hϊs߂̒ۃNXłB
 * <p>
 * @author C/pHeR
 */
abstract public class AbstractTransformer implements ClassFileTransformer {

	/** K[ */
	private static final Logger log = Logger.getLogger(AbstractTransformer.class);

	/** ֘ANXpXiSWTAOSGI Ȃǁj */
	private Set<String> relatedClassPathSet = new HashSet<String>();
	
	/**
	 * oCgR[hϊ܂B
	 * <p>
	 * @see ClassFileTransformer#transform(ClassLoader, String, Class,
	 *          java.security.ProtectionDomain, byte[])
	 */
	final public byte[] transform(
			ClassLoader loader,
			String internalName,
			Class<?> classBeingRedefined,
			ProtectionDomain protectionDomain,
			byte[] bytecode) throws IllegalClassFormatException {
		
		if (loader == null ||
			internalName.startsWith("java/") ||
			internalName.startsWith("javax/") ||
			internalName.startsWith("sun/") ||
			internalName.startsWith("com/sun/") ||
			internalName.startsWith("org/apache/") ||
			internalName.startsWith("jp/sourceforge/mergedoc/pleiades/")) {
			
			// JDK  Pleiades g̓oCgR[hϊȂ
			return null;
		}
		
		try {
			String className = internalName.replace('/', '.');
			return transform(loader, className, protectionDomain, bytecode);
			
		} catch (Throwable e) {
			String msg = "oCgR[hϊɎs܂B" + internalName;
			log.error(msg, e);
			throw new IllegalClassFormatException(msg + " :" + e);
		}
	}
	
	/**
	 * oCgR[hϊ邽߂̃ev[gE\bhłB
	 * <p>
	 * @param loader NXE[_[
	 * @param className NX
	 * @param protectionDomain veNVEhC
	 * @param bytecode oCgR[h
	 * @return ϊ̃oCgR[hBϊKvꍇ nullB
	 * @throws CannotCompileException NXRpCoȂꍇ
	 * @throws NotFoundException NXȂꍇ
	 * @throws IOException o͗Oꍇ
	 */
	abstract protected byte[] transform(
			ClassLoader loader,
			String className,
			ProtectionDomain protectionDomain,
			byte[] bytecode
	) throws CannotCompileException, NotFoundException, IOException;
	
	/**
	 * oCgR[hw肵 CtClass IuWFNg쐬܂B
	 * <p>
	 * @param bytecode oCgR[h
	 * @param protectionDomain veNVEhC
	 * @return CtClass IuWFNg
	 * @throws IOException o͗Oꍇ
	 * @throws NotFoundException NXȂꍇ
	 */
	protected CtClass createCtClass(
			byte[] bytecode,
			ProtectionDomain protectionDomain) throws IOException, NotFoundException {
		
	    ClassPool classPool = new ClassPool();
	    classPool.appendSystemPath();
		URL location = protectionDomain.getCodeSource().getLocation();
		
	    if (location != null) {
	    	
			// NXpXɁA쐬NX܂ރoh̃pXǉ
			File path = new File(location.getPath());
			String classPath = path.getAbsolutePath().replace("%20", " ");
		    classPool.appendClassPath(classPath);
		    
		    // NXpXɊ֘ANXpXǉ
			if (classPath.contains("org.eclipse.swt.") ||
				classPath.contains("mergedoc.jstyle.swt") ||
				classPath.contains("org.eclipse.jface_") ||
				classPath.contains("org.eclipse.osgi_")) {
				
				relatedClassPathSet.add(classPath);
			}
			for (String relatedClassPath : relatedClassPathSet) {
			    classPool.appendClassPath(relatedClassPath);
			}
	    }
		
		// CtClass IuWFNg쐬
        return classPool.makeClass(new ByteArrayInputStream(bytecode));
	}
	
	/**
	 * w肳ꂽ CtClass IuWFNgׂẴRXgN^
	 * \bhLqqOɏo͂܂B
	 * <p>
	 * @param ctClass CtClass IuWFNg
	 */
	protected void logDescriptor(CtClass ctClass) {
		for (CtBehavior behavior : ctClass.getDeclaredBehaviors()) {
			log.info("DESCRIPTOR: " +
					ctClass.getName() + "#" +
					behavior.getName() + " " +
					behavior.getMethodInfo().getDescriptor());
		}
	}
}
